#include <stdio.h>
#include <stdlib.h>
#include <tcl.h>
#include <tk.h>
#include <string.h>

#include "fly.h"

/*
 * Declarations for various library procedures and variables (don't want
 * to include tkInt.h or tkConfig.h here, because people might copy this
 * file out of the Tk source directory to make their own modified versions).
 * - Taken from mkMain.c
 */



/*
 * Global variables used by the main program:
 */

char           *tcl_RcFileName = NULL;  /* Name of a user-specific startup
                                         * script to source if the
                                         * application is being run
                                         * interactively (e.g. "~/.wishrc").
                                         * Set by Tcl_AppInit.  NULL means
                                         * don't source anything ever. */
static int      tty;            /* Non-zero means standard input is a
                                 * terminal-like device.  Zero means it's a
                                 * file. */
static char     errorExitCmd[] = "exit 1";

/*
 * Command-line options:
 */

#if 0
static int      synchronize = 0;

static char    *fileName = NULL;
static char    *name = NULL;
static char    *display = NULL;
static char    *geometry = NULL;
#endif

/*
 * Forward declarations for procedures defined later in this file:
 */

#if 0
static void Prompt _ANSI_ARGS_((Tcl_Interp * interp, int partial));
static void StdinProc _ANSI_ARGS_((ClientData clientData,
                      int mask));
extern int      videoframecmd(ClientData clientData, Tcl_Interp * interp, int argc,
                   char *argv[]);

#endif

int             main(int argc, char **argv)
{
   Tcl_Interp     *interp;      /* Interpreter for this application. */
   ParentWindow   *parentwindow;
   XVisualInfo     visual_info;
   XVisualInfo    *visual_list;
   int             n_visual;
   char           *msg;
   int            *Depth_List;
   int             Depth_count;
   int             Max_Depth;
   int             i;
   char            color_name[256];

   unsigned long   plane_masks[1];
   int             ncolor;
   XColor         *colors;
   unsigned long  *pixels;

   if ((parentwindow = (ParentWindow *) malloc(sizeof(ParentWindow))) == NULL)
   {
      fprintf(stderr, "Out of memory!\n");
      exit(1);
   }

   parentwindow->interp = interp = Tcl_CreateInterp();
   parentwindow->name = strdup("NPAC fly-by demo");

   /*
    * If a display was specified, put it into the DISPLAY environment
    * variable so that it will be available for any sub-processes created by
    * us.
    */
   parentwindow->displayname = NULL;
/*
    if (display != NULL) {
        Tcl_SetVar2(interp, "env", "DISPLAY", displayname, TCL_GLOBAL_ONLY);
    }
*/

   /*
    * Initialize the Tk application.
    */

   if ((parentwindow->tkwin = Tk_CreateMainWindow(interp,
                                                  parentwindow->displayname,
                                                  parentwindow->name, "Tk"))
       == NULL)
   {
      fprintf(stderr, "%s\n", interp->result);
      exit(1);
   }

#if 0
#if 0
   _Xdebug = 1;
#endif
   XSynchronize(Tk_Display(parentwindow->tkwin), True);
#endif

   parentwindow->width = 694;
   parentwindow->height = 749;
   Tk_GeometryRequest(parentwindow->tkwin, parentwindow->width,
                      parentwindow->height);

   parentwindow->display = Tk_Display(parentwindow->tkwin);
#if 0
   parentwindow->visual = Tk_Visual(parentwindow->tkwin);
#endif
   visual_info.screen = Tk_ScreenNumber(parentwindow->tkwin);

   if ((Depth_List = XListDepths(parentwindow->display, visual_info.screen,
                                 &Depth_count)) != NULL)
   {
      Max_Depth = Depth_List[Depth_count - 1];
      for (i = Depth_count - 2; i >= 0; i--)
         if (Max_Depth < Depth_List[i])
            Max_Depth = Depth_List[i];
   }
   else
   {
      fprintf(stderr, "Cannot get depth list\n");
      exit(1);
   }
   
   XFree((char *) Depth_List);

   printf("Screen depth = %d\n", Max_Depth);

   visual_info.depth = Max_Depth;
   if ((visual_list = XGetVisualInfo(parentwindow->display,
                                     VisualScreenMask | VisualDepthMask,
                                     &visual_info, &n_visual)) == NULL)
   {
      fprintf(stderr,"no visual information available\n");
      exit(1);
   }

   fprintf(stderr, "Number of visual = %d\n", n_visual);


   /*
    * Remember various properties of this visual.
    */
   for (i = 0; i < n_visual; i++)
      switch (visual_list[i].class)
      {
         case DirectColor:
            visual_info = visual_list[i];
            fprintf(stderr, "Visual = DirectColor\n");
            break;
         case TrueColor:
            fprintf(stderr, "Visual = TrueColor\n");
            break;
         case PseudoColor:
            fprintf(stderr, "Visual = PseudoColor\n");
            break;
         case StaticColor:
            fprintf(stderr, "Visual = StaticColor\n");
            break;
         case GrayScale:
            fprintf(stderr, "Visual = GrayScale\n");
            break;
         case StaticGray:
            fprintf(stderr, "Visual = StaticGray\n");
            break;
      }

   parentwindow->depth = visual_info.depth;
   parentwindow->visual = visual_info.visual;
   parentwindow->visual_info = visual_info;

   XFree((char *) visual_list);

   parentwindow->colormap = XCreateColormap(parentwindow->display,
      RootWindowOfScreen(Tk_Screen(parentwindow->tkwin)),
      parentwindow->visual, AllocNone);

   Tk_SetWindowVisual(parentwindow->tkwin, parentwindow->visual_info.visual,
                      parentwindow->visual_info.depth, parentwindow->colormap);

   parentwindow->ncolor = ncolor = parentwindow->visual->map_entries;
   parentwindow->colors = colors = (XColor *) ckalloc(ncolor * sizeof(XColor));
   parentwindow->pixels = pixels = (unsigned long *) ckalloc(ncolor
                                                    * sizeof(unsigned long));

   if (!XAllocColorCells(parentwindow->display, parentwindow->colormap,
                         False, plane_masks, 0, pixels, ncolor))
   {
      printf("Allocate color fail\n");
      exit(1);
   }

   printf("ncolor = %d\n", ncolor);
   for (i = 0; i < ncolor; i++)
   {
      sprintf(color_name, "#%06x", pixels[i]);
      printf("color_name[%d] = (%s)\n", i, color_name);
      if (XParseColor(parentwindow->display, parentwindow->colormap,
                      &color_name, &colors[i]))
      {
         colors[i].pixel = pixels[i];
         colors[i].flags = DoRed | DoGreen | DoBlue;
         printf("(Red, Green, Blue) = (%hd, %hd, %hd)\n",
                colors[i].red, colors[i].green, colors[i].blue);
      }
      else
      {
         printf("XParseColor failed\n");
         exit(1);
      }
   }

   Tk_SetWindowColormap(parentwindow->tkwin, parentwindow->colormap);

   /*
    * Make command-line arguments available in the Tcl variables "argc" and
    * "argv".  Also set the "geometry" variable from the geometry specified
    * on the command line.
    */
/*
    if (geometry != NULL) {
        Tcl_SetVar(interp, "geometry", geometry, TCL_GLOBAL_ONLY);
    }
*/

   /*
    * Set the "tcl_interactive" variable.
    */

   Tcl_SetVar(interp, "tcl_interactive", isatty(0) ? "1" : "0",
              TCL_GLOBAL_ONLY);

   /*
    * Initialize tcl and tk
    */
   if (Tcl_Init(interp) == TCL_ERROR)
   {
      fprintf(stderr, "Tcl_Init failed: %s\n", interp->result);
   }
   if (Tk_Init(interp) == TCL_ERROR)
   {
      fprintf(stderr, "Tk_Init failed: %s\n", interp->result);
   }

   if (Photo_Init(interp) == TCL_ERROR)
   {
      fprintf(stderr, "Photo_Init failed: %s\n", interp->result);
   }

   Tcl_CreateCommand(interp, "viewer", (Tcl_CmdProc *) viewercmd,
                     (ClientData *) parentwindow, (Tcl_CmdDeleteProc *) NULL);
   /*
    * Invoke the script specified
    */

   printf("source fly.tcl\n");
   if (Tcl_VarEval(interp, "source fly.tcl", (char *) NULL) == TCL_OK)
   {
      tty = 0;
      fflush(stdout);

      /*
       * Loop infinitely, waiting for commands to execute.  When there are no
       * windows left, Tk_MainLoop returns and we exit.
       */

#if 1
      Tk_MainLoop();
#else
      while (tk_NumMainWindows > 0) {
         Tk_DoOneEvent(0);

         if (newimage == TRUE)
            do_UpdateViewer();
         if (new_view_parameter == TRUE)
            do_compute_view_info();
      }
#endif
      /*
       * Don't exit directly, but rather invoke the Tcl "exit" command. This
       * gives the application the opportunity to redefine "exit" to do
       * additional cleanup.
       */

      Tcl_Eval(interp, "exit");
      exit(0);
   }
   else
   {
      if ((msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY)) == NULL)
         msg = interp->result;
      fprintf(stderr, "%s\n", msg);
      Tcl_Eval(interp, errorExitCmd);
      exit(1);          /* Needed only to prevent compiler warnings. */
   }
}
